﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace Petzold.Books
{
    public class FlipTransition : PageTransition
    {
        FrameworkElement[] pageContainers = new FrameworkElement[3];
        PlaneProjection[] projections = new PlaneProjection[3];

        public override double AnimationDuration
        {
            get { return 500; }
        }

        public override void Attach(Panel containerPanel,
                                    FrameworkElement pageContainer0,
                                    FrameworkElement pageContainer1,
                                    FrameworkElement pageContainer2)
        {
            pageContainers[0] = pageContainer0;
            pageContainers[1] = pageContainer1;
            pageContainers[2] = pageContainer2;

            for (int i = 0; i < 3; i++)
            {
                projections[i] = new PlaneProjection
                {
                    CenterOfRotationX = 0
                };
                pageContainers[i].Projection = projections[i];
            }
        }

        public override void Detach()
        {
            foreach (FrameworkElement pageContainer in pageContainers)
                pageContainer.RenderTransform = null;
        }

        protected override void ShowPageTransition(int baseIndex, double fraction)
        {
            int nextIndex = (baseIndex + 1) % 3;
            int prevIndex = (baseIndex + 2) % 3;

            if (fraction >= 0)
            {
                Canvas.SetZIndex(pageContainers[prevIndex], 1);
                Canvas.SetZIndex(pageContainers[nextIndex], 2);
                Canvas.SetZIndex(pageContainers[baseIndex], 3);

                projections[baseIndex].RotationY = 90 * fraction;
                projections[nextIndex].RotationY = 0;
                projections[prevIndex].RotationY = 0;
            }
            else
            {
                Canvas.SetZIndex(pageContainers[nextIndex], 1);
                Canvas.SetZIndex(pageContainers[baseIndex], 2);
                Canvas.SetZIndex(pageContainers[prevIndex], 3);

                projections[prevIndex].RotationY = 90 * (1 + fraction);
                projections[baseIndex].RotationY = 0;
                projections[nextIndex].RotationY = 0;
            }
        }
    }
}
