﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace Petzold.Books
{
    public class SkewTransition : PageTransition
    {
        FrameworkElement[] pageContainers = new FrameworkElement[3];
        CompositeTransform[] compositeTransforms = new CompositeTransform[3];

        public override double AnimationDuration
        {
            get { return 500; }
        }

        public override void Attach(Panel containerPanel,
                                    FrameworkElement pageContainer0,
                                    FrameworkElement pageContainer1,
                                    FrameworkElement pageContainer2)
        {
            pageContainers[0] = pageContainer0;
            pageContainers[1] = pageContainer1;
            pageContainers[2] = pageContainer2;

            for (int i = 0; i < 3; i++)
            {
                compositeTransforms[i] = new CompositeTransform();
                pageContainers[i].RenderTransform = compositeTransforms[i];
            }
        }

        public override void Detach()
        {
            foreach (FrameworkElement pageContainer in pageContainers)
                pageContainer.RenderTransform = null;
        }

        protected override void ShowPageTransition(int baseIndex, double fraction)
        {
            int nextIndex = (baseIndex + 1) % 3;
            int prevIndex = (baseIndex + 2) % 3;

            if (fraction > -0.5 && fraction < 0.5)
            {
                compositeTransforms[baseIndex].CenterY = pageContainers[baseIndex].ActualHeight;
                compositeTransforms[nextIndex].CenterY = pageContainers[nextIndex].ActualHeight;
                compositeTransforms[prevIndex].CenterY = pageContainers[prevIndex].ActualHeight;

                compositeTransforms[baseIndex].SkewX =
                compositeTransforms[nextIndex].SkewX =
                compositeTransforms[prevIndex].SkewX = 
                    fraction * 360 * Math.Atan2(pageContainers[baseIndex].ActualWidth, 
                                                pageContainers[baseIndex].ActualHeight) / Math.PI;

                compositeTransforms[baseIndex].TranslateX = 0;
                compositeTransforms[nextIndex].TranslateX = compositeTransforms[baseIndex].TranslateX + pageContainers[baseIndex].ActualWidth;
                compositeTransforms[prevIndex].TranslateX = compositeTransforms[baseIndex].TranslateX - pageContainers[prevIndex].ActualWidth;
            }
            else
            {
                compositeTransforms[baseIndex].CenterY = 0;
                compositeTransforms[nextIndex].CenterY = 0;
                compositeTransforms[prevIndex].CenterY = 0;

                compositeTransforms[baseIndex].SkewX =
                compositeTransforms[nextIndex].SkewX =
                compositeTransforms[prevIndex].SkewX =
                    (Math.Sign(fraction) - fraction) * 360 * 
                        Math.Atan2(pageContainers[baseIndex].ActualWidth,
                                   pageContainers[baseIndex].ActualHeight) / Math.PI;

                compositeTransforms[baseIndex].TranslateX = -pageContainers[prevIndex].ActualWidth;
                compositeTransforms[nextIndex].TranslateX = compositeTransforms[baseIndex].TranslateX + pageContainers[baseIndex].ActualWidth;
                compositeTransforms[prevIndex].TranslateX = compositeTransforms[baseIndex].TranslateX - pageContainers[prevIndex].ActualWidth;
            }
        }
    }
}
